import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_story/src/models/story.dart';
import 'package:ocg_story/src/repository/story/base.dart';

///
/// Events
///
abstract class StoryListEvent {}

class StoryListRefreshEvent extends StoryListEvent {
  final bool isRefresh;

  StoryListRefreshEvent({this.isRefresh = false});
}

class StoryListDeleteEvent extends StoryListEvent {
  final String storyId;

  StoryListDeleteEvent(this.storyId);
}

// TODO Testing
class StoryListUpdateEvent extends StoryListEvent {
  final Story story;

  StoryListUpdateEvent(this.story);
}

abstract class StoryListState {}

class StoryListIdleState extends StoryListState {}

class StoryListLoadingState extends StoryListState {}

class StoryListLoadedState extends StoryListState {
  final List<ProfileStories> stories;

  StoryListLoadedState(this.stories);
}

class StoryListErrorState extends StoryListState {
  final String error;

  StoryListErrorState(this.error);
}

class StoryListBloc extends Bloc<StoryListEvent, StoryListState> {
  final StoryRepository storyRepository;
  StoryListBloc(this.storyRepository) : super(StoryListIdleState());

  @override
  Stream<StoryListState> mapEventToState(StoryListEvent event) async* {
    if (event is StoryListRefreshEvent) {
      if (!event.isRefresh && state is StoryListLoadedState) {
        yield state;
      } else {
        yield StoryListLoadingState();
      }
      AppResponse<List<ProfileStories>> storiesResponse =
          await storyRepository.getFollowing();
      if (storiesResponse.isSuccess) {
        yield StoryListLoadedState(storiesResponse.data);
      } else {
        yield StoryListErrorState(storiesResponse.error);
      }
    }
    if (event is StoryListDeleteEvent) {
      if (state is StoryListLoadedState) {
        int mainIndex = -1;
        var profileStories = (state as StoryListLoadedState).stories.toList();
        for (var i = 0; i < profileStories.length; i++) {
          ProfileStories colleciton = profileStories[i];
          int index = colleciton.stories
              .indexWhere((element) => element.id == event.storyId);
          if (index >= 0) {
            List<Story> stories = colleciton.stories.toList();
            stories.removeAt(index);
            colleciton = colleciton.copyWith(stories: stories);
            profileStories[i] = colleciton;
            if (stories.length == 0) {
              mainIndex = i;
            }
            break;
          }
        }
        if (mainIndex >= 0) {
          profileStories.removeAt(mainIndex);
        }
        yield StoryListLoadedState(profileStories);
      }
    }
    if (event is StoryListUpdateEvent) {
      if (state is StoryListLoadedState) {
        var profileStories = (state as StoryListLoadedState).stories.toList();
        var profile = event.story?.profile;
        int index =
            profileStories.indexWhere((element) => element.profile == profile);
        if (index >= 0) {
          var stories = profileStories[index].stories.toList();
          if (stories.contains(event.story)) {
            stories.remove(event.story);
          }
          stories.add(event.story);
          profileStories[index] =
              profileStories[index].copyWith(stories: stories);
        }
        yield StoryListLoadedState(profileStories);
      }
    }
  }
}
